#r "nuget: arcIO.NET, 0.0.1"
#r "nuget: ISADotNet.QueryModel, 0.5.4"
#r "nuget: BioFSharp.IO, 2.0.0-preview.3"
#r "nuget: FSharp.Stats, 0.4.9"
#r "nuget: Plotly.NET, 3.0.1"
#r "nuget: Deedle, 3.0.0-beta.1"
#r "nuget: DeedleExtensions, 0.2.0"
open FSharp.Stats
open Plotly.NET
open Deedle
open BioFSharp.IO
open DeedleExtensions


open ISADotNet

let arcPath = __SOURCE_DIRECTORY__ + @"\..\..\"

let p,a = arcIO.Net.Assay.readByName arcPath "CDJ5Proteomics"
let qa = QueryModel.QStudy.fromAssay a

qa.ProtocolNames

let allSamples = 
    qa.LastNodes()
    |> Set.ofSeq

let getBioRep fN = 
    qa.ValuesOf(fN,ProtocolName = "Growth").WithName("biological replicate").First.ValueWithUnitText
    |> int          


let getGenotype fN :string= 
    qa.ValuesOf(fN,ProtocolName = "GenotypeLib").WithName("Genotype").First.ValueWithUnitText
    |> string    


let t2 = getGenotype (allSamples |> Array.ofSeq |> Array.item 4)

///
let InP  = __SOURCE_DIRECTORY__ + @"\..\..\runs\3_CDJ5_MSFraggerCombined\MSFraggerCombined.tsv" 

///////////////////////////////////////////////////////
/// Raw Data
///////////////////////////////////////////////////////
let schema = 
    allSamples
    |> Seq.map (fun x -> sprintf "%s = float" x)
    |> String.concat ","

///
let quantValues :Frame<string,string> = 
    Frame.ReadCsv(path = InP, hasHeaders=true, separators="\t",schema= schema)
    |> Frame.indexRows "Protein" 
    |> Frame.mapValues (fun x -> if x > 0. then x else nan)
    // |> Frame.sortRowsBy "Intensity" (fun k -> -k)

quantValues.Print()

quantValues
|> Frame.dropSparseRows
|> Frame.countRows

// quantValues
// |> Frame.dropSparseRows
// |> Frame.getNumericCols 
// |> Series.observations
// |> Seq.sortBy (fun (k,s) -> getGenotype k)
// |> Seq.map (fun (k,s) -> 
//     s  
//     |> Series.values 
//     |> Chart.BoxPlot
//     |> Chart.withTraceInfo (getGenotype k + "_" + (getBioRep k|>string))
//     )
// |> Chart.combine
// |> Chart.show


quantValues
// |> Frame.dropSparseRows
|> Frame.getNumericCols 
|> Series.observations
|> Seq.sortBy (fun (k,s) -> getGenotype k)
|> Seq.map (fun (k,s) -> 
    (getGenotype k + "_" + (getBioRep k|>string)), s |> Stats.sum
    )
|> Chart.Column
|> Chart.withSize(1200.,1000.)
|> Chart.show


quantValues
|> Frame.dropSparseRows
|> Frame.getNumericCols 
|> Series.observations
|> Seq.sortBy (fun (k,s) -> getGenotype k)
|> Seq.map (fun (k,s) -> 
    (getGenotype k + "_" + (getBioRep k|>string)), s |> Stats.sum
    )
|> Chart.Column
|> Chart.withSize(1200.,1000.)
|> Chart.show
     

quantValues
// |> Frame.dropSparseRows
|> Frame.getNumericCols 
|> Series.observations
|> Seq.sortBy (fun (k,s) -> getGenotype k)
|> Seq.map (fun (k,s) -> 
    (getGenotype k + "_" + (getBioRep k|>string)), s |> Series.values |> Seq.map float |> Seq.median
    )
|> Chart.Column
|> Chart.withSize(1200.,1000.)
|> Chart.show

quantValues
|> Frame.dropSparseRows
|> Frame.getNumericCols 
|> Series.observations
|> Seq.sortBy (fun (k,s) -> getGenotype k)
|> Seq.map (fun (k,s) -> 
    (getGenotype k + "_" + (getBioRep k|>string)), s |> Series.values |> Seq.map float |> Seq.median
    )
|> Chart.Column
|> Chart.withSize(1200.,1000.)
|> Chart.show



let getTurboIDLabel fN = 
    qa.ValuesOf(fN,ProtocolName = "GenotypeLib").WithName("gene identifier").First.ValueWithUnitText
    |> string          



let turboIDs = 
    quantValues
    // |> Frame.mapValues (fun v -> if v > 0. then v else nan)
    |> Frame.getNumericCols
    |> Series.map (fun k s -> 
        let label = getTurboIDLabel k
        let m = s |> Series.tryGet label
        match m with 
        | Some x -> k,x
        | _ -> k,0.
        )
    |> Series.values
    |> Chart.Column
    // |> Chart.withTraceName "turboID"
    // |> Chart.withY_AxisStyle "LFQ"

turboIDs
|> Chart.show


let turboIDabundances = 
    quantValues
    // |> Frame.mapValues (fun v -> if v > 0. then v else nan)
    |> Frame.getNumericCols
    |> Series.map (fun k s -> 
        let label = getTurboIDLabel k
        let m = s |> Series.tryGet label
        match m with 
        | Some x -> (getGenotype k + "_" + (getBioRep k|>string)),x
        | _ -> (getGenotype k + "_" + (getBioRep k|>string)),0.
        )
    |> Series.values
    |> Array.ofSeq


let sumQuant = 
    quantValues
    // |> Frame.dropSparseRows
    |> Frame.getNumericCols 
    |> Series.observations
    |> Seq.sortBy (fun (k,s) -> getGenotype k)
    |> Seq.map (fun (k,s) -> 
        (getGenotype k + "_" + (getBioRep k|>string)), s |> Stats.sum
        )
    |> Array.ofSeq

sumQuant
|> Array.map (fun (k,s) ->
        k.Split('_').[1],
        (
        s, 
        turboIDabundances
        |> Array.find (fun (k2,s2) -> 
            k = k2)
        |> snd
        )
    )
|> Array.groupBy (fst)
|> Array.map (fun (k,(d)) -> 
    d
    |> Array.map snd 
    |> Chart.Point
    |> Chart.withTraceInfo k
    )
|> Chart.combine
|> Chart.show


let medianQuant = 
    quantValues
    |> Frame.dropSparseRows
    |> Frame.getNumericCols 
    |> Series.observations
    |> Seq.sortBy (fun (k,s) -> getGenotype k)
    |> Seq.map (fun (k,s) -> 
        (getGenotype k + "_" + (getBioRep k|>string)), s |> Stats.median
        )
    |> Array.ofSeq

medianQuant
|> Array.map (fun (k,s) ->
        k.Split('_').[1],
        (
        s, 
        turboIDabundances
        |> Array.find (fun (k2,s2) -> 
            k = k2)
        |> snd
        )
    )
|> Array.groupBy (fst)
|> Array.map (fun (k,(d)) -> 
    d
    |> Array.map snd 
    |> Chart.Point
    |> Chart.withTraceInfo k
    )
|> Chart.combine
|> Chart.show


module Global = 

    /// As used by Deseq2, see: https://github.com/hbctraining/DGE_workshop/blob/master/lessons/02_DGE_count_normalization.md 
    module MedianOfRatios = 

        /// expects data frame with shape frame<sampleIds,GeneIds>
        let medianOfRatiosWide (data:Frame<string,string>) =
            //data frame with shape frame<sampleIds,GeneIds>
            let ratios = 
                data
                |> Frame.dropSparseCols
                |> Frame.getNumericCols
                |> Series.map (fun geneID s -> 
                    let geometricMeanAcrossAllSamples = 
                        s.Values 
                        |> FSharp.Stats.Seq.meanGeometric
                    let ratioPerSample = 
                        s
                        |> Series.map (fun x sample ->  sample / geometricMeanAcrossAllSamples)
                    ratioPerSample
                    )
                |> Frame.ofColumns
            // Series of shape: Series<sampleId,correctionFactor:float>
            let correctionFactorPerSample = 
                ratios
                |> Frame.transpose
                |> Stats.median
            Chart.Column(correctionFactorPerSample |> Series.observations)
            |> Chart.withXAxisStyle "SampleId"
            |> Chart.withXAxisStyle "Correction Factor."
             |> Chart.show
            // data frame with frame<sampleIds,GeneIds>
            // contains gene data divided by the sample wise estimated correction factor
            let correctedData = 
                data
                |> Frame.getNumericCols 
                |> Series.map (fun geneID s -> 
                    s / correctionFactorPerSample
                    )
                |> Frame.ofColumns
            correctedData

        /// expects data frame with shape frame<GeneIds,sampleIds>
        let medianOfRatios (data:Frame<string,string>) =
            data
            |> Frame.transpose
            |> medianOfRatiosWide
            |> Frame.transpose


let normalized = 
    quantValues
    |> Global.MedianOfRatios.medianOfRatios            


normalized


normalized
// |> Frame.dropSparseRows
|> Frame.getNumericCols 
|> Series.observations
|> Seq.sortBy (fun (k,s) -> getGenotype k)
|> Seq.map (fun (k,s) -> 
    (getGenotype k + "_" + (getBioRep k|>string)), s |> Stats.sum
    )
|> Chart.Column
|> Chart.withSize(1200.,1000.)
|> Chart.show


normalized
|> Frame.dropSparseRows
|> Frame.getNumericCols 
|> Series.observations
|> Seq.sortBy (fun (k,s) -> getGenotype k)
|> Seq.map (fun (k,s) -> 
    (getGenotype k + "_" + (getBioRep k|>string)), s |> Stats.sum
    )
|> Chart.Column
|> Chart.withSize(1200.,1000.)
|> Chart.show
     

normalized
// |> Frame.dropSparseRows
|> Frame.getNumericCols 
|> Series.observations
|> Seq.sortBy (fun (k,s) -> getGenotype k)
|> Seq.map (fun (k,s) -> 
    (getGenotype k + "_" + (getBioRep k|>string)), s |> Series.values |> Seq.map float |> Seq.median
    )
|> Chart.Column
|> Chart.withSize(1200.,1000.)
|> Chart.show

normalized
|> Frame.dropSparseRows
|> Frame.getNumericCols 
|> Series.observations
|> Seq.sortBy (fun (k,s) -> getGenotype k)
|> Seq.map (fun (k,s) -> 
    (getGenotype k + "_" + (getBioRep k|>string)), s |> Series.values |> Seq.map float |> Seq.median
    )
|> Chart.Column
|> Chart.withSize(1200.,1000.)
|> Chart.show



let turboIDabundancesN = 
    normalized
    // |> Frame.mapValues (fun v -> if v > 0. then v else nan)
    |> Frame.getNumericCols
    |> Series.map (fun k s -> 
        let label = getTurboIDLabel k
        let m = s |> Series.tryGet label
        match m with 
        | Some x -> (getGenotype k + "_" + (getBioRep k|>string)),x
        | _ -> (getGenotype k + "_" + (getBioRep k|>string)),0.
        )
    |> Series.values
    |> Array.ofSeq

let sumQuantN = 
    normalized
    // |> Frame.dropSparseRows
    |> Frame.getNumericCols 
    |> Series.observations
    |> Seq.sortBy (fun (k,s) -> getGenotype k)
    |> Seq.map (fun (k,s) -> 
        (getGenotype k + "_" + (getBioRep k|>string)), s |> Stats.sum
        )
    |> Array.ofSeq

sumQuantN
|> Array.map (fun (k,s) ->
        k.Split('_').[1],
        (
        s, 
        turboIDabundancesN
        |> Array.find (fun (k2,s2) -> 
            k = k2)
        |> snd
        )
    )
|> Array.groupBy (fst)
|> Array.map (fun (k,(d)) -> 
    d
    |> Array.map snd 
    |> Chart.Point
    |> Chart.withTraceInfo k
    )
|> Chart.combine
|> Chart.show

let medianQuantN = 
    normalized
    |> Frame.dropSparseRows
    |> Frame.getNumericCols 
    |> Series.observations
    |> Seq.sortBy (fun (k,s) -> getGenotype k)
    |> Seq.map (fun (k,s) -> 
        (getGenotype k + "_" + (getBioRep k|>string)), s |> Stats.median
        )
    |> Array.ofSeq


medianQuantN
|> Array.map (fun (k,s) ->
        k.Split('_').[1],
        (
        s, 
        turboIDabundancesN
        |> Array.find (fun (k2,s2) -> 
            k = k2)
        |> snd
        )
    )
|> Array.groupBy (fst)
|> Array.map (fun (k,(d)) -> 
    d
    |> Array.map snd 
    |> Chart.Point
    |> Chart.withTraceInfo k
    )
|> Chart.combine
|> Chart.show    