#load "annotate.fsx"
open System
open Argu
open Annotate


/// Argument handling
[<AutoOpen>]
module Arguments = 
    type AnnotationArgs =
        | [<Unique>] [<Mandatory>] [<AltCommandLine("-i")>] InputPath                   of path:string
        | [<Unique>] [<Mandatory>] [<AltCommandLine("-o")>] OutputPath                  of path:string
        | [<Unique>] [<Mandatory>] [<AltCommandLine("-c")>] IdentifierColumnHeader      of string
        | [<Unique>] [<Mandatory>] [<AltCommandLine("-t")>] ColumnSeparator             of string
        |            [<Mandatory>] [<AltCommandLine("-a")>] Annotation                  of string
        | [<Unique>]               [<AltCommandLine("-x")>] AnnotationSeparator         of string
        | [<Unique>]               [<AltCommandLine("-y")>] IdentifierSeparator         of string
        interface IArgParserTemplate with
            member this.Usage =
                match this with
                | InputPath x                   -> "relative input file path in /arc/"
                | OutputPath x                  -> "relative output file path in /arc/runs/"
                | IdentifierColumnHeader x      -> "column header of identifier column file to annotate"
                | ColumnSeparator x             -> "column separator (tab , ; ...)"
                | Annotation x                  -> "annotationHeader"
                | AnnotationSeparator x         -> "multiple annotation separator"
                | IdentifierSeparator x         -> "multiple identifier separator"
    
    let annotationArgParser = ArgumentParser.Create<AnnotationArgs>(programName = "annotateChlamy.fsx")
    
    let annotationR = 
        fsi.CommandLineArgs
        |> Array.skip 1 
        |> fun args -> annotationArgParser.ParseCommandLine(args) 
   
    // arguments are converted
    let inputPath                    = annotationR.GetResult(InputPath)//@"/arc/assays/" + annotationR.GetResult(InputPath)
    let outputFilePath               = annotationR.GetResult(OutputPath)
    let columnHeader                 = annotationR.GetResult(IdentifierColumnHeader)
    let separator                    = getSeparator (annotationR.GetResult(ColumnSeparator))
    let mappings                     = annotationR.GetResults(Annotation) |> Array.ofList
    let multipleIdentifierSeparator  = getSeparator (annotationR.GetResult(IdentifierSeparator,";"))
    let multipleAnnotationSeparator  = getSeparator (annotationR.GetResult(AnnotationSeparator,"|"))
    
    if separator = "," then 
        failwithf "\n\nWARNING: Annotation descriptions may contain comma and therefore corrupts column separation!\n"


let truncID (id : string) = (id.Split([|".p";".t"|],StringSplitOptions.None)).[0]

let mappingFrame = Mapping.readMapping (__SOURCE_DIRECTORY__ + @"\..\..\studies\CDJ5\resources\chlamy_jgi55.txt") "\t" "Identifier"
Data.annotateAndWriteData mappingFrame inputPath separator columnHeader truncID mappings multipleIdentifierSeparator multipleAnnotationSeparator outputFilePath

///Full annotation of chlamydomonas test file
//dotnet fsi annotateChlamy.fsx -i assays/chlamydomonas/dataset/chlamydomonas.txt -o creResult/creFullAnnotationdd.tsv -c "Protein IDs truncated" -t tab -a Synonym -a MapMan -a MapManDescription -a GO -a GODescription -a Localization 
///Symbol annotation of chlamydomonas test file
//dotnet fsi annotateChlamy.fsx -i assays/chlamydomonas/dataset/chlamydomonas.txt -o creResult/creFullAnnotationdd.tsv -c "Protein IDs truncated" -t tab -a Synonym 

