#r "nuget: arcIO.NET, 0.0.1"
#r "nuget: ISADotNet.QueryModel, 0.5.4"
#r "nuget: BioFSharp.IO, 2.0.0-preview.3"
#r "nuget: FSharp.Stats, 0.4.12-preview.1"
#r "nuget: Plotly.NET, 3.0.1"
// #r "nuget: Deedle, 3.0.0-beta.1"
#r "nuget: Deedle"
#r "nuget: DeedleExtensions, 0.2.0"
#r "nuget: metaXa, 0.1.0"
open MetaXa
open MetaFrame
open Stats
open FSharp.Stats
open Plotly.NET
open Deedle
open BioFSharp.IO
open DeedleExtensions
open ISADotNet

type UseControl = 
    | C4
    | C13
    | Both
5+5
// let batch = 2
// let conditionGenotype = "UVM_CDJ5_WT_TurboID"
let control1 = "UVM_Cher4"
let control2 = "UVM_Cher13"
let inP  = __SOURCE_DIRECTORY__ + @"\..\..\runs\4_CDJ5_MSFraggerCombined_annotated\MSFraggerCombined_annotated.tsv" 
let proteinID = "Protein" 
let inAssay = "CDJ5Proteomics"
let conditionGenotype = "UVM_TEF_TurboID"//"UVM_CDJ5_WT_TurboID"
let batch = 2//2
let normGlobal = true
let fdr = 0.01
// let doAnalysis controlsToUse normGlobal normByBait batch conditionGenotype control1 control2 inP proteinID inAssay = 
/////////////////////////////////////////////////////
// / params
/////////////////////////////////////////////////////
// let conditionGenotype = conditionGenotype//"UVM_CDJ5_WT_TurboID"
// let control1 = control1//"UVM_Cher4"
// let control2 = control2//"UVM_Cher13"
// let inP  = inP//__SOURCE_DIRECTORY__ + @"\..\..\runs\4_CDJ5_MSFraggerCombined_annotated\MSFraggerCombined_annotated.tsv" 
// let proteinID = proteinID//"Protein" 
// let inAssay = inAssay//"CDJ5Proteomics"
// let normGlobal = normGlobal
/////////////////////////////////////////////////////
// Isa
/////////////////////////////////////////////////////
let arcPath = __SOURCE_DIRECTORY__ + @"\..\..\"

let p,a = arcIO.Net.Assay.readByName arcPath inAssay

let qa = QueryModel.QStudy.fromAssay a

let allSamples = 
    qa.LastNodes()
    |> Set.ofSeq

let getBioRep fN = 
    qa.ValuesOf(fN,ProtocolName = "Growth").WithName("biological replicate").First.ValueWithUnitText
    |> int       

let getGenotype fN = 
    qa.ValuesOf(fN,ProtocolName = "GenotypeLib").WithName("Genotype").First.ValueWithUnitText
    |> string   

let getBatch fN = 
    qa.ValuesOf(fN,ProtocolName = "Growth").WithName("Batch").First.ValueWithUnitText
    |> int    

let getTurboIDIdentifier fN = 
    qa.ValuesOf(fN,ProtocolName = "GenotypeLib").WithName("gene identifier").First.ValueWithUnitText
    |> string        

/////////////////////////////////////////////////////
// / Raw Data
/////////////////////////////////////////////////////
let schema = 
    allSamples
    |> Seq.map (fun x -> sprintf "%s=float" x)
    |> String.concat ","

// /
let all :Frame<string,string> = 
    Frame.ReadCsv(path = inP, hasHeaders=true, separators="\t",schema= schema)
    |> Frame.indexRows proteinID 
    |> Frame.mapValues (fun x -> if x > 0. then x else nan)
    
let quantValues :Frame<string,string> = 
    all
    |> Frame.sliceCols allSamples

let allColumnsCondition = 
    quantValues.ColumnKeys 
    |> Seq.filter (fun x -> getGenotype x = conditionGenotype) 
    |> Array.ofSeq

let allColumnsCher4Control = 
    quantValues.ColumnKeys 
    |> Seq.filter (fun x -> getGenotype x = control1 && getBatch x = batch) 
    |> Array.ofSeq
 
let allColumnsCher13Control = 
    quantValues.ColumnKeys 
    |> Seq.filter (fun x -> getGenotype x = control2 && getBatch x = batch) 
    |> Array.ofSeq

let allColumnsControl = 
    (Array.append allColumnsCher4Control allColumnsCher13Control)

let quantValuesOfInterest = 
    quantValues
    |> Frame.sliceCols (Array.concat [|allColumnsCondition;allColumnsControl|])

quantValuesOfInterest.ColumnKeys 
  
/////////////////////////////////////////////////////
// Normalized Data
/////////////////////////////////////////////////////
let normalizedAndTransposed = 
    if normGlobal then 
        let fst = 
            quantValuesOfInterest
            |> Frame.sliceCols allColumnsCondition
            |> Frame.transpose
            |> Stats.Normalization.Global.MedianOfRatios.medianOfRatiosWide
        let snd = 
            quantValuesOfInterest
            |> Frame.sliceCols allColumnsControl
            |> Frame.transpose
            |> Stats.Normalization.Global.MedianOfRatios.medianOfRatiosWide
        Frame.merge fst snd
    else
        quantValuesOfInterest
        |> Frame.transpose


/////////////////////////////////////////////////////
// Imputation
/////////////////////////////////////////////////////
let descByISa :Frame<string,string> = 
    [
        "Genotype",
        allSamples
        |> Seq.map (fun x -> x,getGenotype x)
        |> Series.ofObservations
        "biological replicate",
        allSamples
        |> Seq.map (fun x -> x,getBioRep x |> string)
        |> Series.ofObservations
    ]
    |> Frame.ofColumns
    |> Frame.filterRows (fun k s -> [|allColumnsCondition;allColumnsControl|] |> Array.concat |> Array.contains k )
descByISa.Print()

let fittingSpline lambda (data:seq<(float*float)>) =
    let x,y = data |> Array.ofSeq |> Array.unzip
    let xV,yV = x |> vector, y |> vector
    let basisP = // xV |> Array.ofSeq
        [|
            for i = 0 to xV.Length-1 do 
                if i = 0 then 
                    xV.[i]
                elif i % 50 = 0 then xV.[i]
                elif i = xV.Length-1 then xV.[xV.Length-1]
        |]

    FSharp.Stats.Fitting.Spline.smoothingSpline (data |> Array.ofSeq) basisP lambda

let fittingSpline' (data:seq<(float*float)>) =
    fittingSpline 1000000000000. data

let sortingInner = 
    (fun (s:ObjectSeries<string>) -> 
        let repIdx = s.GetAs<int>("biological replicate") 
        repIdx 
    )

let sortingOuter = 
    (fun (s:ObjectSeries<string>) -> 
        let g = s.GetAs<string>("Genotype")
        if g = conditionGenotype then 0
        elif g = control1 then 1
        elif g = control2 then 2
        else failwith "Genotype not supported by design"        
    )

// First we create a new !sorted! description based on a grouping created specifing the columns in the original description which are used to create a grouping. 
let descriptionSorted =  Group.groupAndSortByFactorLevels sortingInner sortingOuter ["Genotype";] (String.concat "_") descByISa
descriptionSorted.Print() 

// Then, we sort and group the data based on the new description
let dataSorted = Group.groupRowsByGroupedFactors descriptionSorted normalizedAndTransposed
dataSorted.Print() 
(dataSorted |> Frame.sliceCols ["Cre-1.g000035.t1.p1"; "Cre-1.g2716952.t1.p1";"sp|P05413|FABPH_HUMAN"]).Print()

// let centerData (imputedData:Frame<string,string>) = 
//     let median = 
//         imputedData
//         |> Stats.mean
//     let stabw = 
//         imputedData 
//         |> Stats.stdDev
//     imputedData
//     |> Frame.mapValues (fun x -> 
//         (x-median) / stabw    
//         )
        
// Then, we perform an imputation by simple gaussian sampling, if no value was recorded we sample out of the lowest 0.1 quantile of the data set.
let imputeSamplingCondition = 
    Stats.Imputation.SimpleSampling.computeColumnWiseUsing Distributions.Continuous.Normal.Sample 3 (Some 0.1) (fittingSpline') 
        (dataSorted |> Frame.filterRows (fun k s -> allColumnsCondition |> Array.contains (snd k))) 
    // |> centerData

let imputeSamplingControl = 
    Stats.Imputation.SimpleSampling.computeColumnWiseUsing Distributions.Continuous.Normal.Sample 3 (Some 0.1) (fittingSpline') 
        (dataSorted |> Frame.filterRows (fun k s -> (Array.append allColumnsCher4Control allColumnsCher13Control) |> Array.contains (snd k))) 
    // |> centerData

let imputeSampling = 
    Frame.merge imputeSamplingCondition imputeSamplingControl

(imputeSampling |> Frame.sliceCols ["Cre-1.g000035.t1.p1"; "Cre-1.g2716952.t1.p1";"sp|P05413|FABPH_HUMAN"]).Print()

let imputeSamplingT = 
    imputeSampling.Transpose()
    |> Frame.mapValues log2
    
// to separate control and factor sets (sample1 and sample2, respectively), 
// the datasets are chunked by the number of replicates (here: triplicates -> chunkBySize 3).
let (preData1,preData2) :float[][] * float [][]=  
    imputeSamplingT
    |> Frame.mapRows (fun k s -> 
        allColumnsCondition 
        |> Array.map (fun c -> s.GetAs<float>(c)),
        [|
        allColumnsCher4Control 
        |> Array.map (fun c -> s.GetAs<float>(c))
        allColumnsCher13Control 
        |> Array.map (fun c -> s.GetAs<float>(c))
        |]
        |> Array.concat
        )
    |> Series.values
    |> Array.ofSeq
    |> Array.unzip
// let (preData1Norm,preData2Norm) =  centerData preData1, centerData preData2
// After chunking, the datasets are separated by factor and the rowkey is added to the data for later identification.
let rowheader :string[] = imputeSamplingT.RowKeys |> Seq.toArray
let data1 = Array.zip rowheader  preData1//preData1//
let data2 = Array.zip rowheader  preData2//preData2//

let res = FSharp.Stats.Testing.SAM.twoClassUnpaired 100 fdr  data2 data1  (System.Random(1234))

res.NegSigBioitem.Length

res.PosSigBioitem.Length

#load "SamPlotting.fsx"
open SamPlotting
createSAMChart res
|> Chart.show

let missingValuesCondition =
    all
    |> Frame.sliceCols allColumnsCondition
    |> Frame.mapRows (fun k s -> 
        allColumnsCondition
        |> Array.map (fun x -> 
            s.GetAs<float>(x,nan)
           )
        |> Array.filter (fun x -> nan.Equals x)
        |> Array.length
        )

let missingValuesControl =
    all
    |> Frame.sliceCols allColumnsControl
    |> Frame.mapRows (fun k s -> 
        allColumnsControl
        |> Array.map (fun x -> 
            s.GetAs<float>(x,nan)
           )
        |> Array.filter (fun x -> nan.Equals x)
        |> Array.length
        )

let fc = 
    imputeSamplingT
    |> Frame.mapRows (fun k s -> 
        let m1 = allColumnsCondition |> Seq.map (fun x -> s.GetAs<float>(x,nan)) |> Seq.mean
        let m2 = allColumnsControl   |> Seq.map (fun x -> s.GetAs<float>(x,nan)) |> Seq.mean
        m1-m2
    )

let meanAbundance = 
    imputeSamplingT
    |> Frame.transpose
    |> Stats.mean

let stdev = 
    imputeSamplingT
    |> Frame.sliceCols allColumnsCondition 
    |> Frame.transpose
    |> Stats.stdDev

let qValue = 
    [|res.PosSigBioitem;res.NegSigBioitem;res.NonSigBioitem|]
    |> Array.concat
    |> Array.map (fun x -> x.ID,x.QValue)
    |> Series.ofObservations

let IsCandidate = 
    [|res.PosSigBioitem;res.NegSigBioitem;res.NonSigBioitem|]
    |> Array.concat
    |> Array.map (fun x -> x.ID,x.QValue < fdr )
    |> Series.ofObservations

    
let finPlot = 
    imputeSamplingT
    |> Frame.addCol "MeanR" fc
    |> Frame.addCol "MeanAbundance" meanAbundance
    |> Frame.addCol "StDevR" stdev
    |> Frame.addCol "qValue" qValue
    |> Frame.addCol "missingValuesCondition" missingValuesCondition
    |> Frame.addCol "missingValuesControl" missingValuesControl
    |> Frame.addCol "IsCandidate" IsCandidate
    |> Frame.addCol "ProtName" (all.GetColumn<string>("Synonym"))
    
#load "Plotting.fsx"
open Plotting

plotMA ["CDJ5";"LON";"PSBQLD";"PGRL1"] finPlot
|> Chart.saveHtml (__SOURCE_DIRECTORY__ + @"\..\..\runs\5_Analysis_SAM\MAPLot_1PercentFDRCutoff.")


plotHisto finPlot
|> Chart.show

plotVulcano ["CDJ5";"LON";"PSBQLD";"PGRL1"] finPlot
|> Chart.show

finPlot.Print()
   
let fin = 
    imputeSamplingT
    |> Frame.addCol "log2FC_Condition_Minus_Control" fc
    |> Frame.addCol "AverageLog2Abundance" meanAbundance
    |> Frame.addCol "qValue" qValue
    |> Frame.addCol "missingValuesCondition" missingValuesCondition
    |> Frame.addCol "missingValuesControl" missingValuesControl
    |> Frame.addCol "IsCandidate" IsCandidate
    |> Frame.addCol "ProtName" (all.GetColumn<string>("Synonym"))
    
let toSave = Frame.join JoinKind.Outer fin (Frame.sliceCols ["Synonym"; "MapMan";"MapManDescription"; "GO"; "GODescription"] all)    

toSave.SaveCsv(__SOURCE_DIRECTORY__ + @"\..\..\runs\5_Analysis_SAM\Analysis_complete.tsv",includeRowKeys=true,keyNames=["Protein"],separator='\t')

