#r "nuget: arcIO.NET, 0.0.1"
#r "nuget: ISADotNet.QueryModel, 0.5.4"
#r "nuget: BioFSharp.IO, 2.0.0-preview.3"
#r "nuget: FSharp.Stats, 0.4.9"
#r "nuget: Plotly.NET, 3.0.1"
// #r "nuget: Deedle, 3.0.0-beta.1"
#r "nuget: Deedle"
#r "nuget: DeedleExtensions, 0.2.0"
open FSharp.Stats
open Plotly.NET
open Deedle
open BioFSharp.IO
open DeedleExtensions


let plotMA highlightList (f:Frame<string,string>) = 
    let dataAll = 
        f
        |> Frame.mapRows (fun k s -> 
            let a = s.GetAs<float>("MeanAbundance",nan)
            let r = s.GetAs<float>("MeanR",nan)
            let stDev = s.GetAs<float>("StDevR",nan)
            let l = 
                let x = s.GetAs<string>("ProtName","")
                if x = "" then 
                    "" 
                else 
                    x
            let c = s.GetAs<bool>("IsCandidate")
            {|
                    ProtName         = l
                    MeanAbundance    = a
                    MeanR            = r
                    StDevR           = stDev
                    IsCandidate      = c
            |}
            )
        |> Series.values
        |> Array.ofSeq
    
    let all = 
        let x = dataAll |> Seq.map (fun x -> x.MeanR) //|> Seq.map log10
        let y = dataAll |> Seq.map (fun x -> x.MeanAbundance) //|> Seq.map log10
        let label = dataAll |> Seq.map (fun x -> x.ProtName)
        // let xError = dataAll |> Seq.map (fun x -> x.StDevR)
        // Chart.Point(x,y,"",Labels = label)
        Chart.Point(y,x)
        // |> Chart.withXErrorStyle (Array=xError,Symmetric=true)
        // |> Chart.withXErrorStyle(xError,Thickness = 0.5)
    let candidates =
        let dataFiltered = dataAll |> Array.filter (fun x -> x.IsCandidate)  
        let x = dataFiltered |> Seq.map (fun x -> x.MeanR) //|> Seq.map log10
        let y = dataFiltered |> Seq.map (fun x -> x.MeanAbundance) //|> Seq.map log10
        let labels = dataFiltered |> Seq.map (fun x -> x.ProtName)
        // let xError = dataFiltered |> Seq.map (fun x -> x.StDevR)
        Chart.Point(
        y,x,
        Name="points",
        MultiText=labels,
        TextPosition=StyleParam.TextPosition.TopRight
        )
        // |> Chart.withYErrorStyle (Array=xError,Symmetric=true,Thickness=0.5,Width=3.)
        |> Chart.withTraceInfo "candidates"
        // |> Chart.withXErrorStyle(xError,Thickness = 0.5)
    let candidatesNoStabw =
        let dataFiltered = dataAll |> Array.filter (fun x -> x.IsCandidate)  
        let x = dataFiltered |> Seq.map (fun x -> x.MeanR) //|> Seq.map log10
        let y = dataFiltered |> Seq.map (fun x -> x.MeanAbundance) //|> Seq.map log10
        let labels = dataFiltered |> Seq.map (fun x -> x.ProtName)
        // let xError = dataFiltered |> Seq.map (fun x -> x.StDevR)
        Chart.Point(
        y,x,
        Name="points"
        // MultiText=labels,
        // TextPosition=StyleParam.TextPosition.TopRight
        )
        // |> Chart.withXErrorStyle (Array=xError,Symmetric=true,Thickness=0.5,Width=3.)
        |> Chart.withTraceInfo "candidatesNoStabw"
        // |> Chart.withXErrorStyle(xError,Thickness = 0.5)
    let candidates' =
        let dataFiltered = dataAll |> Array.filter (fun x -> highlightList |> List.contains x.ProtName )  
        let x = dataFiltered |> Seq.map (fun x -> x.MeanR) //|> Seq.map log10
        let y = dataFiltered |> Seq.map (fun x -> x.MeanAbundance) //|> Seq.map log10
        let labels = dataFiltered |> Seq.map (fun x -> x.ProtName)
        // let xError = dataFiltered |> Seq.map (fun x -> x.StDevR)
        Chart.Point(
        y,x,
        Name="points",
        MultiText=labels,
        TextPosition=StyleParam.TextPosition.TopRight
        )
        // |> Chart.withYErrorStyle (Array=xError,Symmetric=true,Thickness=0.5,Width=3.)
        |> Chart.withTraceInfo "candidates Reci"
        // |> Chart.withXErrorStyle(xError,Thickness = 0.5) 
    [
        all
        // candidates
        candidatesNoStabw
        candidates'

    ]
    |> Chart.combine
    |> Chart.withYAxisStyle "log2(condition)-log2(control)"
    |> Chart.withXAxisStyle "log2(Mean intensity)"

let plotVulcano highlightList f = 
    let dataAll = 
        f
        |> Frame.mapRows (fun k s -> 
            let a = s.GetAs<float>("MeanAbundance",nan)
            let r = s.GetAs<float>("MeanR",nan)
            let stDev = s.GetAs<float>("StDevR",nan)
            let qValue = s.GetAs<float>("qValue",nan)
            let l = 
                let x = s.GetAs<string>("ProtName","")
                if x = "" then 
                    "" 
                else 
                    x
            let c = s.GetAs<bool>("IsCandidate")
            {|
                    ProtName         = l
                    MeanAbundance    = a
                    MeanR            = r
                    StDevR           = stDev
                    IsCandidate      = c
                    QVal = qValue
            |}
            )
        |> Series.values
        |> Array.ofSeq
    
    let all = 
        let y = dataAll |> Seq.map (fun x -> x.MeanR)
        let x = 
            dataAll 
            |> Seq.map (fun x -> x.QVal) 
            |> Seq.map (fun x -> 
                let res = -log10 x
                if infinity.Equals res then 4. else res
                )
        let label = dataAll |> Seq.map (fun x -> x.ProtName)
        let xError = 
            dataAll |> Seq.map (fun x -> x.StDevR)
        Chart.Point(y,x)
    let candidatesNoStabw =
        let dataFiltered = dataAll |> Array.filter (fun x -> x.IsCandidate)  
        let y = dataFiltered |> Seq.map (fun x -> x.MeanR) 
        let x = 
            dataFiltered 
            |> Seq.map (fun x -> x.QVal) 
            |> Seq.map (fun x -> 
                let res = -log10 x
                if infinity.Equals res then 4. else res
                )
        let labels = dataFiltered |> Seq.map (fun x -> x.ProtName)
        let xError = dataFiltered |> Seq.map (fun x -> x.StDevR)
        Chart.Point(
        y,x,
        Name="points",
        MultiText=labels
        )
        |> Chart.withTraceInfo "candidatesNoStabw"
    let candidates' =
        let dataFiltered = dataAll |> Array.filter (fun x -> highlightList |> List.contains x.ProtName )  
        let y = dataFiltered |> Seq.map (fun x -> x.MeanR) 
        let x = 
            dataFiltered 
            |> Seq.map (fun x -> x.QVal) 
            |> Seq.map (fun x -> 
                let res = -log10 x
                if infinity.Equals res then 4. else res
                )
        let labels = dataFiltered |> Seq.map (fun x -> x.ProtName)
        Chart.Point(
        y,x,
        Name="points",
        MultiText=labels,
        TextPosition=StyleParam.TextPosition.TopRight
        )
        |> Chart.withTraceInfo "candidates Reci"
    [
        all
        candidatesNoStabw
        candidates'

    ]
    |> Chart.combine
    |> Chart.withYAxisStyle "log2(condition)-log2(control)"
    |> Chart.withXAxisStyle "log2(Mean intensity)"

let plotHisto (f:Frame<string,string>) =
    let dataAll = 
        f
        |> Frame.mapRows (fun k s -> 
            let a = s.GetAs<float>("MeanAbundance",nan)
            let r = s.GetAs<float>("MeanR",nan)
            let stDev = s.GetAs<float>("StDevR",nan)
            let l = ""//s.GetAs<string>("ProtName","")
            let c = s.GetAs<bool>("IsCandidate")
            {|
                    ProtName         = l
                    MeanAbundance    = a
                    MeanR            = r
                    StDevR           = stDev
                    IsCandidate      = c
            |}
            )
        |> Series.values
        |> Array.ofSeq
    let histo =     
        let data = dataAll |> Seq.map (fun x -> x.MeanR)   |> Seq.filter (fun x -> nan.Equals x |> not)|> Array.ofSeq
        let bw = FSharp.Stats.Distributions.Bandwidth.nrd0 data 
        let binned = FSharp.Stats.Distributions.Frequency.create  (bw) data |> Map.toSeq
        Chart.Column binned
        |> Chart.withXAxisStyle "R3"
    histo









