#r "nuget: FSharp.Stats, 0.4.10"
#r "nuget: Plotly.NET, 3.0.1"
// #r "nuget: Deedle, 3.0.0-beta.1"
#r "nuget: Deedle"
#r "nuget: DeedleExtensions, 0.2.0"
open Deedle
open FSharp.Stats
open Plotly.NET
open FSharp.Stats.Testing
open SAM
open Plotly.NET.StyleParam

#load "SamPlotting.fsx"
open SamPlotting

let df:Frame<string,string> = 
    Frame.ReadCsv(__SOURCE_DIRECTORY__ + @"/TestDataSAM.txt",hasHeaders=true,separators = "\t")
    // here, the name of A1 is needed. 
    |> Frame.indexRows "gene"
    

df.Print()
// get Rowkeys as Array
let rowheader :string[] = df.RowKeys |> Seq.toArray

let c1 = [|"1";"12";"13"|]

let c2 = [|"2";"22";"23"|]//;"1";"2";"1"]
// to separate control and factor sets (sample1 and sample2, respectively), 
// the datasets are chunked by the number of replicates (here: triplicates -> chunkBySize 3).
let (preData1,preData2) :float[][] * float [][]=  
    df
    |> Frame.mapRows (fun k s -> 
        c1 
        |> Array.map (fun c -> s.GetAs<float>(c)),
        [|
        c2 
        |> Array.map (fun c -> s.GetAs<float>(c))
        c2 
        |> Array.map (fun c -> s.GetAs<float>(c))
        |]
        |> Array.concat
        )
    |> Series.values
    |> Array.ofSeq
    |> Array.unzip

// After chunking, the datasets are separated by factor and the rowkey is added to the data for later identification.
let data1 = Array.zip rowheader preData1 
let data2 = Array.zip  rowheader preData2 

let corrected1 = 
    let medCorrect = 
        medianCentering data1 
    Array.zip rowheader medCorrect

let corrected2 = 
    let medCorrect = 
        medianCentering data2
    Array.zip rowheader medCorrect
 
let res = FSharp.Stats.Testing.SAM.twoClassUnpaired 100 0.05 data1 data2 (System.Random(1234))


createSAMChart res
|> Chart.show