#r "nuget: arcIO.NET, 0.0.1"
#r "nuget: ISADotNet.QueryModel, 0.5.4"
#r "nuget: BioFSharp.IO, 2.0.0-preview.3"
#r "nuget: FSharp.Stats, 0.4.9"
#r "nuget: Plotly.NET, 3.0.1"
// #r "nuget: Deedle, 3.0.0-beta.1"
#r "nuget: Deedle"
#r "nuget: DeedleExtensions, 0.2.0"
#r "nuget: metaXa, 0.1.0"
open MetaXa
open MetaFrame
open Stats
open FSharp.Stats
open Plotly.NET
open Deedle
open BioFSharp.IO
open DeedleExtensions
open ISADotNet

type UseControl = 
    | C4
    | C13
    | Both
5+5
// let batch = 2
// let conditionGenotype = "UVM_CDJ5_WT_TurboID"
let control1 = "UVM_Cher4"
let control2 = "UVM_Cher13"
let inP  = __SOURCE_DIRECTORY__ + @"\..\..\runs\4_CDJ5_MSFraggerCombined_annotated\MSFraggerCombined_annotated.tsv" 
let proteinID = "Protein" 
let inAssay = "CDJ5Proteomics"
let conditionGenotype = "UVM_TEF_TurboID"//"UVM_CDJ5_WT_TurboID"
let batch = 2//2
let normGlobal = true
// let doAnalysis controlsToUse normGlobal normByBait batch conditionGenotype control1 control2 inP proteinID inAssay = 
/////////////////////////////////////////////////////
// / params
/////////////////////////////////////////////////////
// let conditionGenotype = conditionGenotype//"UVM_CDJ5_WT_TurboID"
// let control1 = control1//"UVM_Cher4"
// let control2 = control2//"UVM_Cher13"
// let inP  = inP//__SOURCE_DIRECTORY__ + @"\..\..\runs\4_CDJ5_MSFraggerCombined_annotated\MSFraggerCombined_annotated.tsv" 
// let proteinID = proteinID//"Protein" 
// let inAssay = inAssay//"CDJ5Proteomics"
// let normGlobal = normGlobal
/////////////////////////////////////////////////////
// Isa
/////////////////////////////////////////////////////
let arcPath = __SOURCE_DIRECTORY__ + @"\..\..\"

let p,a = arcIO.Net.Assay.readByName arcPath inAssay

let qa = QueryModel.QStudy.fromAssay a

let allSamples = 
    qa.LastNodes()
    |> Set.ofSeq

let getBioRep fN = 
    qa.ValuesOf(fN,ProtocolName = "Growth").WithName("biological replicate").First.ValueWithUnitText
    |> int       

let getGenotype fN = 
    qa.ValuesOf(fN,ProtocolName = "GenotypeLib").WithName("Genotype").First.ValueWithUnitText
    |> string   

let getBatch fN = 
    qa.ValuesOf(fN,ProtocolName = "Growth").WithName("Batch").First.ValueWithUnitText
    |> int    

let getTurboIDIdentifier fN = 
    qa.ValuesOf(fN,ProtocolName = "GenotypeLib").WithName("gene identifier").First.ValueWithUnitText
    |> string        

/////////////////////////////////////////////////////
// / Raw Data
/////////////////////////////////////////////////////
let schema = 
    allSamples
    |> Seq.map (fun x -> sprintf "%s=float" x)
    |> String.concat ","

// /
let all :Frame<string,string> = 
    Frame.ReadCsv(path = inP, hasHeaders=true, separators="\t",schema= schema)
    |> Frame.indexRows proteinID 
    |> Frame.mapValues (fun x -> if x > 0. then x else nan)
    
let quantValues :Frame<string,string> = 
    all
    |> Frame.sliceCols allSamples

let allColumnsCondition = 
    quantValues.ColumnKeys 
    |> Seq.filter (fun x -> getGenotype x = conditionGenotype) 
    |> Array.ofSeq

let allColumnsCher4Control = 
    quantValues.ColumnKeys 
    |> Seq.filter (fun x -> getGenotype x = control1 && getBatch x = batch) 
    |> Array.ofSeq
 
let allColumnsCher13Control = 
    quantValues.ColumnKeys 
    |> Seq.filter (fun x -> getGenotype x = control2 && getBatch x = batch) 
    |> Array.ofSeq

let quantValuesOfInterest = 
    quantValues
    |> Frame.sliceCols (Array.concat [|allColumnsCondition;allColumnsCher4Control;allColumnsCher13Control|])
/////////////////////////////////////////////////////
// Normalized Data
/////////////////////////////////////////////////////
let normalizedAndTransposed = 
    if normGlobal then 
        quantValues
        |> Frame.transpose
        |> Stats.Normalization.Global.MedianOfRatios.medianOfRatiosWide
    else
        quantValues
        |> Frame.transpose


/////////////////////////////////////////////////////
// Imputation
/////////////////////////////////////////////////////
let descByISa :Frame<string,string> = 
    [
        "Genotype",
        allSamples
        |> Seq.map (fun x -> x,getGenotype x)
        |> Series.ofObservations
        "biological replicate",
        allSamples
        |> Seq.map (fun x -> x,getBioRep x |> string)
        |> Series.ofObservations
    ]
    |> Frame.ofColumns
    |> Frame.filterRows (fun k s -> [|allColumnsCondition;allColumnsCher4Control;allColumnsCher13Control|] |> Array.concat |> Array.contains k )
descByISa.Print()

let fittingSpline lambda (data:seq<(float*float)>) =
    let x,y = data |> Array.ofSeq |> Array.unzip
    let xV,yV = x |> vector, y |> vector
    let basisP = // xV |> Array.ofSeq
        [|
            for i = 0 to xV.Length-1 do 
                if i = 0 then 
                    xV.[i]
                elif i % 50 = 0 then xV.[i]
                elif i = xV.Length-1 then xV.[xV.Length-1]
        |]

    FSharp.Stats.Fitting.Spline.smoothingSpline (data |> Array.ofSeq) basisP lambda

let fittingSpline' (data:seq<(float*float)>) =
    fittingSpline 10000000000. data

let sortingInner = 
    (fun (s:ObjectSeries<string>) -> 
        let repIdx = s.GetAs<int>("biological replicate") 
        repIdx 
    )


let sortingOuter = 
    (fun (s:ObjectSeries<string>) -> 
        let g = s.GetAs<string>("Genotype")
        if g = conditionGenotype then 0
        elif g = control1 then 1
        elif g = control2 then 2
        else failwith "Genotype not supported by design"        
    )

// First we create a new !sorted! description based on a grouping created specifing the columns in the original description which are used to create a grouping. 
let descriptionSorted =  Group.groupAndSortByFactorLevels sortingInner sortingOuter ["Genotype";] (String.concat "_") descByISa
descriptionSorted.Print() 

// Then, we sort and group the data based on the new description
let dataSorted = Group.groupRowsByGroupedFactors descriptionSorted normalizedAndTransposed
dataSorted.Print() 
(dataSorted |> Frame.sliceCols ["Cre-1.g000035.t1.p1"; "Cre-1.g2716952.t1.p1";"sp|P05413|FABPH_HUMAN"]).Print()

// Then, we perform an imputation by simple gaussian sampling, if no value was recorded we sample out of the lowest 0.1 quantile of the data set.
let imputeSampling = Stats.Imputation.SimpleSampling.computeColumnWiseUsing Distributions.Continuous.Normal.Sample 2 (Some 0.1) (fittingSpline') dataSorted 

(imputeSampling |> Frame.sliceCols ["Cre-1.g000035.t1.p1"; "Cre-1.g2716952.t1.p1";"sp|P05413|FABPH_HUMAN"]).Print()



let calcRep norm (f:Frame<string,string>) (rep:int) = 
    let condC = allColumnsCondition |> Array.find (fun x -> getBioRep x = rep)
    let cher4Control = allColumnsCher4Control |> Array.find (fun x -> getBioRep x = rep)
    let cher13Control = allColumnsCher13Control |> Array.find (fun x -> getBioRep x = rep)

    let CGE = f.GetColumn<float>(condC)    
    let _13 = f.GetColumn<float>(cher13Control)  
    let _4  = f.GetColumn<float>(cher4Control)  

    let cgeBaitFactor   = CGE |> Series.get (getTurboIDIdentifier condC)
    let _13BaitFactor   = _13 |> Series.get (getTurboIDIdentifier cher13Control)
    let _4BaitFactor    = _4 |> Series.get (getTurboIDIdentifier cher4Control)

    let NormAndLog2_Condition = 
        CGE 
        |> Series.mapValues (fun (prot) -> 
                    let scaled1 = 
                        if norm then 
                            prot / cgeBaitFactor
                        else
                            prot
                    log2 scaled1
            )


    let NormAndLog2_Cher13_1 = 
        _13
        |> Series.mapValues (fun (prot) -> 
                    let scaled1 = 
                        if norm then 
                            prot / _13BaitFactor
                        else
                            prot
                    log2 scaled1
            )

    
    let NormAndLog2_Cher4_2 = 
        _4
        |> Series.mapValues (fun (prot) -> 
                    let scaled1 = 
                        if norm then 
                            prot / _4BaitFactor
                        else
                            prot
                    log2 scaled1
            )  
    [
        (rep |> string) + "_NormAndLog10_Condition" ,NormAndLog2_Condition;
        (rep |> string) + "_NormAndLog10_Cher13_1"  ,NormAndLog2_Cher13_1;
        (rep |> string) + "_NormAndLog10_Cher4_2"   ,NormAndLog2_Cher4_2;
        (rep |> string) + "_abundance_cge",CGE;
        (rep |> string) + "_abundance_13",_4;
        (rep |> string) + "_abundance_4",_4
    ]
    |> Frame.ofColumns


let calculateStats controlsToUse norm (f:Frame<string,string>) :Frame<string,string>=
    
    // Todo: get replicates to iter from sample description
    let r3s = 
        [calcRep norm f 1;calcRep norm f 2;calcRep norm f 3;calcRep norm f 4]
        |> Frame.mergeAll 

    let aCols = r3s.ColumnKeys |> Seq.filter (fun x -> x.Contains "abundance")
    let NormAndLog10_ConditionCols = r3s.ColumnKeys |> Seq.filter (fun x -> x.Contains "NormAndLog10_Condition") |> Array.ofSeq
    let NormAndLog10_Cher13_1Cols = r3s.ColumnKeys |> Seq.filter (fun x -> x.Contains "NormAndLog10_Cher13_1") |> Array.ofSeq
    let NormAndLog10_Cher4_2Cols = r3s.ColumnKeys |> Seq.filter (fun x -> x.Contains "NormAndLog10_Cher4_2") |> Array.ofSeq
        
    let meanRatioNoSparse =  
        r3s
        |> Frame.dropSparseRows
        |> Frame.mapRows (fun k s -> 
                NormAndLog10_ConditionCols 
                |> Array.map (fun ck -> 
                    let valCondition = s.GetAs<float>(ck,nan)
                    let partnerColCher13 = Array.find (fun (x:string) -> ck.Split('_').[0] = x.Split('_').[0]) NormAndLog10_Cher13_1Cols
                    let valCher13 = s.GetAs<float>(partnerColCher13,nan)
                    let partnerColCher4 = Array.find (fun (x:string) -> ck.Split('_').[0] = x.Split('_').[0]) NormAndLog10_Cher4_2Cols
                    let valCher4 = s.GetAs<float>(partnerColCher4,nan)
                    [|
                        valCondition-valCher13;
                        valCondition-valCher4
                    |]
                    )
                |> Array.concat
                |> Array.filter (fun x -> nan.Equals(x) |> not)
                |> Seq.mean
            )
        |> Series.values
        |> Array.ofSeq

    let cutOff = 
        let borders =
            meanRatioNoSparse
            |> Array.ofSeq
            |> FSharp.Stats.Testing.Outliers.tukey 1.5
        borders.Upper
    let highestRatio :float = meanRatioNoSparse |> Seq.max
    let lowestRatio  :float = meanRatioNoSparse |> Seq.min
    let stats = 
        r3s
        |> Frame.mapRows (fun k s -> 
                let asOfProt = aCols |> Seq.map (fun ck -> s.GetAs<float>(ck,nan)) |> Seq.filter (fun x -> nan.Equals x |> not)
                let cgeAbundances = aCols |> Seq.filter (fun x -> x.Contains "cge") |> Seq.map (fun ck -> s.GetAs<float>(ck,nan)) |> Seq.filter (fun x -> nan.Equals x |> not)                
                let mCherryAbundances = aCols |> Seq.filter (fun x -> x.Contains "cge" |> not) |> Seq.map (fun ck -> s.GetAs<float>(ck,nan)) |> Seq.filter (fun x -> nan.Equals x |> not)                
                let meanAbundance = asOfProt |> Seq.map log2 |> Seq.mean
                let rsCher13 = 
                    NormAndLog10_ConditionCols 
                    |> Array.map (fun ck -> 
                        let valCondition = s.GetAs<float>(ck,nan)
                        let partnerColCher13 = Array.find (fun (x:string) -> ck.Split('_').[0] = x.Split('_').[0]) NormAndLog10_Cher13_1Cols
                        let valCher13 = s.GetAs<float>(partnerColCher13,nan)
                        valCondition-valCher13
                    )
                    |> Array.filter (fun x -> nan.Equals x |> not)
                let rsCher4 = 
                    NormAndLog10_ConditionCols 
                    |> Array.map (fun ck -> 
                        let valCondition = s.GetAs<float>(ck,nan)
                        let partnerColCher4 = Array.find (fun (x:string) -> ck.Split('_').[0] = x.Split('_').[0]) NormAndLog10_Cher4_2Cols
                        let valCher4 = s.GetAs<float>(partnerColCher4,nan)
                        valCondition-valCher4
                        )
                    |> Array.filter (fun x -> nan.Equals x |> not)
                let nRatios = (rsCher4 |> Seq.length) + (rsCher13 |> Seq.length)
                let nQuantifiedInCGE = cgeAbundances |> Seq.length
                let nQuantifiedInMCherry = mCherryAbundances |> Seq.length
                let meanR = 
                    if nRatios > 0 then
                        match controlsToUse with 
                        | Both -> 
                            Seq.mean (Array.concat [|rsCher13;rsCher4|])
                        | C13 ->
                            Seq.mean (Array.concat [|rsCher13;|])
                        | C4 ->
                            Seq.mean (Array.concat [|rsCher4;|])
                    elif nQuantifiedInCGE > 0 && nQuantifiedInMCherry = 0 then 
                        highestRatio 
                    elif nQuantifiedInMCherry > 0 && nQuantifiedInCGE = 0 then 
                        lowestRatio     
                    else nan           
                let stDevR = 
                    if nRatios > 1 then
                        match controlsToUse with 
                        | Both -> 
                            Seq.stDev (Array.concat [|rsCher13;rsCher4|])
                        | C13 ->
                            Seq.stDev (Array.concat [|rsCher13;|])
                        | C4 ->
                            Seq.stDev (Array.concat [|rsCher4;|])
                    else 
                        0.
                let isCandidate = meanR > cutOff
                {|
                    Index           = k
                    IsCandidate     = isCandidate
                    MeanAbundance   = meanAbundance
                    MeanR           = meanR
                    StDevR          = stDevR
                    NRatios         = nRatios
                    NQuantifiedInCGE = nQuantifiedInCGE
                |}
            )
        |> Series.values
    stats
    |> Frame.ofRecords
    |> Frame.indexRows "Index"

let normByBait = true
let controlsToUse = UseControl.Both 
let normalized = imputeSampling.Transpose()

let resLog = calculateStats controlsToUse normByBait  normalized

let resres =
    resLog
    |> Frame.addCol "ProtName" (all.GetColumn<string>("Synonym"))

#load "Plotting.fsx"
open Plotting

plotRatiosVsMeanAbundanceSpec ["CDJ5";"LON";"PSBQLN";"PGRL1"] resres
|> Chart.withTitle (sprintf "resNoNorm, globalnorm:%A, control:%A" true conditionGenotype)
|> Chart.show
