﻿
open System
open Argu 
open App.Annotate
open App.Arguments
open ISADotNet
open ISADotNet.QueryModel
open Deedle
open App.MetaxaTemp
open Plotly.NET
open FSharp.Stats
open MetaXa
open App.Plotting
open App.SamPlotting

[<EntryPoint>]
let main args =

    let annotationArgParser = ArgumentParser.Create<WorkflowArgs>(programName = "annotateChlamy.fsx")
    printfn "%A" args
    let annotationR = 
        args
        // |> Array.skip 1 
        |> fun args -> annotationArgParser.ParseCommandLine(args) 
   
    // arguments are converted
    let inputPath                    = annotationR.GetResult(InputPath)//@"/arc/assays/" + annotationR.GetResult(InputPath)
    let arcPath                      = annotationR.GetResult(ArcPath)
    let runID                        = annotationR.GetResult(RunID)
    let mappingFilePath              = annotationR.GetResult(MappingFilePath)
    // let outputFilePath               = annotationR.GetResult(OutputPath)
    let columnHeader                 = 
        match annotationR.TryGetResult(IdentifierColumnHeader) with 
        | None -> "Protein"
        | Some x -> x
    let fdr                 = 
        match annotationR.TryGetResult(FDR) with 
        | None -> 0.01
        | Some x -> x |> float
    let performNorm = 
        match annotationR.TryGetResult (PerformNormalization) with 
        | None -> false
        | Some x -> true


    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////
    ////////////////////////////////////////
    // Init OutDir 
    ////////////////////////////////////////
    ////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    let runID = runID
    System.IO.Directory.CreateDirectory runID
    |> ignore
    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////
    ////////////////////////////////////////
    // Init MetaData lookup functions
    ////////////////////////////////////////
    ////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////

    let qi = 
        arcIO.NET.Investigation.fromArcFolder arcPath
        |> QueryModel.QInvestigation.fromInvestigation

    // qi.ProtocolNames

    let allSamples =         
        qi.LastNodes
        |> Array.ofSeq

    // allSamples
    // |> Array.map (fun x -> x.Name)

    let getBioRep (fN:QNode) = 
        match qi.ValuesOf(fN,ProtocolName = "Growth").WithName("biological replicate").Values.Head with
        | QueryModel.ISAValue.Parameter x -> x.Value.Value.AsString 
        | _ -> failwith "no biorep please add"

    // allSamples
    // |> Array.map getBioRep

    let getBioRepGroup (fN:QNode) = 
        match qi.ValuesOf(fN,ProtocolName = "Growth").WithName("biological replicate group").Values.Head with
        | QueryModel.ISAValue.Characteristic x -> x.Value.Value.AsString 
        | _ -> failwith "no biorep please add"

    // allSamples
    // |> Array.map getBioRepGroup

    let qNodesOfInterest = 
        allSamples

    let sampleNameToNode =
        qNodesOfInterest
        |> Array.map (fun x -> x.Name, x)
        |> Map.ofArray

    let qSampleNameToIntensityColumn fN = 
        fN + " Intensity" 

    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////
    ////////////////////////////////////////
    // Execute Annotation
    ////////////////////////////////////////
    ////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    let mappingFrame = Mapping.readMapping mappingFilePath "\t" "Identifier"
    mappingFrame.Print()
    let truncID (id : string) = (id.Split([|".p";".t"|],System.StringSplitOptions.None)).[0]
    let outPathA = runID + "/AnnotatedResult.tsv"

    Data.annotateAndWriteData 
        mappingFrame 
        inputPath 
        "\t" 
        columnHeader 
        truncID 
        [|"Synonym";"MapMan"; "MapManDescription"; "GO"; "GODescription"; "Localization";"Synonym_Phytozome_61";"GeneName_Phytozome_61"; "Description_Phytozome_61"|]
        ";"
        "|"
        outPathA
    printfn "Finished Annotation" 

    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////
    ////////////////////////////////////////
    // Prepare Analysis
    ////////////////////////////////////////
    ////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    let doAnalysis (allDataForAnalysis:Frame<string,string>) dataForAnalysis (allControlSamples:QNode []) baitGroup (allBaitSamples:QNode []) = //controlsToUse normGlobal normByBait batch conditionGenotype control1 control2 inP proteinID inAssay =    
        printfn "Start Analysis: %s vs Control" baitGroup
        let allSamples = [|allBaitSamples;allControlSamples|] |> Array.concat

        let allColsOfIBait = 
            allBaitSamples
            |> Array.map (fun x -> qSampleNameToIntensityColumn x.Name)     
        
        let allColsOfIControl = 
            allControlSamples
            |> Array.map (fun x -> qSampleNameToIntensityColumn x.Name)     
        
        let quantValues :Frame<string,string> = 
            dataForAnalysis
            |> Frame.sliceCols ([|allColsOfIBait;allColsOfIControl|] |> Array.concat)

        /////////////////////////////////////////////////////
        // Normalized Data
        /////////////////////////////////////////////////////
        printfn "Normalize Data"
        /// 
        
        let normalizedAndTransposed, abundanceChart, corrFChart, corrAbundanceChart = 
            // if normGlobal then 
                let fst,ai,cor,aCorr = 
                    quantValues
                    |> Frame.sliceCols allColsOfIBait
                    |> Frame.transpose
                    |> Global.MedianOfRatios.medianOfRatiosWide
                let snd,aiCon,corCon,aCorrCon = 
                    quantValues
                    |> Frame.sliceCols allColsOfIControl
                    |> Frame.transpose
                    |> Global.MedianOfRatios.medianOfRatiosWide
                Frame.merge fst snd, 
                Chart.combine
                    [
                        ai
                        |> Chart.withTraceInfo "Bait"
                        aiCon
                        |> Chart.withTraceInfo "Control"
                    ], 
                Chart.combine
                    [
                        cor
                        |> Chart.withTraceInfo "Bait"
                        corCon
                        |> Chart.withTraceInfo "Control"
                    ],
                Chart.combine
                    [
                        aCorr
                        |> Chart.withTraceInfo "Bait"
                        aCorrCon
                        |> Chart.withTraceInfo "Control"
                    ]
            // else
            //     quantValuesOfInterest
            //     |> Frame.transpose
        
        /////////////////////////////////////////////////////
        // Imputation
        /////////////////////////////////////////////////////
        printfn "Impute Data"
        let descByISa :Frame<string,string> = 
            [
                "BioRepGroup",
                allSamples
                |> Seq.map (fun x -> qSampleNameToIntensityColumn x.Name, getBioRepGroup x)
                |> Series.ofObservations
                "biological replicate",
                allSamples
                |> Seq.map (fun x -> qSampleNameToIntensityColumn x.Name, getBioRep x |> string)
                |> Series.ofObservations
            ]
            |> Frame.ofColumns

        // let fittingSpline lambda (data:seq<(float*float)>) =
        //     let x,y = data |> Array.ofSeq |> Array.unzip
        //     let xV,yV = x |> vector, y |> vector
        //     let basisP = // xV |> Array.ofSeq
        //         [|
        //             for i = 0 to xV.Length-1 do 
        //                 if i = 0 then 
        //                     xV.[i]
        //                 elif i % 50 = 0 then xV.[i]
        //                 elif i = xV.Length-1 then xV.[xV.Length-1]
        //         |]

        //     FSharp.Stats.Fitting.Spline.smoothingSpline (data |> Array.ofSeq) basisP lambda

        let fitCV (data:seq<(float*float)>)  =
            // fittingSpline 1000000000000. data
            let m = 
                data 
                |> Seq.map snd 
                |> Seq.median
            fun x -> m

        let sortingInner = 
            (fun (s:ObjectSeries<string>) -> 
                let repIdx = s.GetAs<int>("biological replicate") 
                repIdx 
            )

        let sortingOuter = 
            (fun (s:ObjectSeries<string>) -> 
                let g = s.GetAs<string>("BioRepGroup")
                if g = baitGroup then 0 else 1
                // else failwith "Genotype not supported by design"        
            )

        // First we create a new !sorted! description based on a grouping created specifing the columns in the original description which are used to create a grouping. 
        let descriptionSorted =  Group.groupAndSortByFactorLevels sortingInner sortingOuter ["BioRepGroup";] (String.concat "_") descByISa

        // Then, we sort and group the data based on the new description
        let dataSorted = Group.groupRowsByGroupedFactors descriptionSorted normalizedAndTransposed

        // Then, we perform an imputation by simple gaussian sampling, if no value was recorded we sample out of the lowest 0.1 quantile of the data set.
        let imputeSamplingCondition = 
            Imputation.SimpleSampling.computeColumnWiseUsing Distributions.Continuous.Normal.Sample 3 (Some 0.01) (fitCV) 
                (dataSorted |> Frame.filterRows (fun k s -> allColsOfIBait |> Array.contains (snd k))) 
            |> Frame.mapValues log2

        let imputeSamplingControl = 
            Imputation.SimpleSampling.computeColumnWiseUsing Distributions.Continuous.Normal.Sample 3 (Some 0.01) (fitCV) 
                (dataSorted |> Frame.filterRows (fun k s -> allColsOfIControl |> Array.contains (snd k))) 
            |> Frame.mapValues log2

        let imputeSampling = 
            Frame.merge imputeSamplingCondition imputeSamplingControl

        let imputeSamplingT = 
            imputeSampling.Transpose()

        let preImpChart = 
            let ck = 
                quantValues.ColumnKeys
                |> Array.ofSeq
            quantValues
            |> Frame.mapValues log2
            |> Frame.toJaggedArray
            |> Matrix.ofJaggedArray
            |> Matrix.mapiCols (fun i c -> 
                Chart.Histogram c
                |> Chart.withTraceInfo ck.[i]
                )
            |> Chart.combine
            |> Chart.withXAxisStyle "log2 Abundance"
        let afterImpChart = 
            let ck = 
                imputeSamplingT.ColumnKeys
                |> Array.ofSeq
            imputeSamplingT
            |> Frame.toJaggedArray
            |> Matrix.ofJaggedArray
            |> Matrix.mapiCols (fun i c -> 
                Chart.Histogram c
                |> Chart.withTraceInfo ck.[i]
                )
            |> Chart.combine
            |> Chart.withXAxisStyle "log2 Abundance"
        /////////////////////////////////////////////////////
        // Norm By Quantile Normalization
        /////////////////////////////////////////////////////
        printfn "perform Quantile Normalization:%b" performNorm
        
        let normFrame =
            if performNorm then  
                let normM = 
                    imputeSamplingT
                    |> Frame.toJaggedArray
                    |> Matrix.ofJaggedArray
                    |> FSharp.Stats.Signal.Normalization.quantile
                    |> Matrix.toArray2D
                normM
                |> Frame.ofArray2D
                |> Frame.indexRowsWith imputeSamplingT.RowKeys
                |> Frame.indexColsWith imputeSamplingT.ColumnKeys
            else 
                imputeSamplingT
        let preNormChart = 
            let ck = 
                normFrame.ColumnKeys
                |> Array.ofSeq
            imputeSamplingT
            |> Frame.toJaggedArray
            |> Matrix.ofJaggedArray
            |> Matrix.mapiCols (fun i c -> 
                Chart.Histogram c
                |> Chart.withTraceInfo ck.[i]
                )
            |> Chart.combine
            |> Chart.withXAxisStyle "log2 Abundance"
        let afterNormChart = 
            let ck = 
                normFrame.ColumnKeys
                |> Array.ofSeq
            normFrame
            |> Frame.toJaggedArray
            |> Matrix.ofJaggedArray
            |> Matrix.mapiCols (fun i c -> 
                Chart.Histogram c
                |> Chart.withTraceInfo ck.[i]
                )
            |> Chart.combine
            |> Chart.withXAxisStyle "log2 Abundance"

        /////////////////////////////////////////////////////
        // Testing
        /////////////////////////////////////////////////////
        printfn "Perform Testing"
        let (preData1,preData2) :float[][] * float [][]=  
            normFrame
            |> Frame.mapRows (fun k s -> 
                allColsOfIBait 
                |> Array.map (fun c -> s.GetAs<float>(c)),
                [|
                allColsOfIControl 
                |> Array.map (fun c -> s.GetAs<float>(c))
                |]
                |> Array.concat
                )
            |> Series.values
            |> Array.ofSeq
            |> Array.unzip

        let rowheader :string[] = normFrame.RowKeys |> Seq.toArray
        let data1 = Array.zip rowheader  preData1//preData1//
        let data2 = Array.zip rowheader  preData2//preData2//

        let res = FSharp.Stats.Testing.SAM.twoClassUnpaired 100 fdr  data2 data1  (System.Random(1234))  
        /////////////////////////////////////////////////////
        // Summary Statistics
        /////////////////////////////////////////////////////
        printfn "Compute Summary Statistics"
        let missingValuesCondition =
            quantValues
            |> Frame.sliceCols allColsOfIBait
            |> Frame.mapRows (fun k s -> 
                allColsOfIBait
                |> Array.map (fun x -> 
                    s.GetAs<float>(x,nan)
                )
                |> Array.filter (fun x -> nan.Equals x)
                |> Array.length
                )

        let missingValuesControl =
            quantValues
            |> Frame.sliceCols allColsOfIControl
            |> Frame.mapRows (fun k s -> 
                allColsOfIControl
                |> Array.map (fun x -> 
                    s.GetAs<float>(x,nan)
                )
                |> Array.filter (fun x -> nan.Equals x)
                |> Array.length
                )

        let fc = 
            normFrame
            |> Frame.mapRows (fun k s -> 
                let m1 = allColsOfIBait |> Seq.map (fun x -> s.GetAs<float>(x,nan)) |> Seq.mean
                let m2 = allColsOfIControl   |> Seq.map (fun x -> s.GetAs<float>(x,nan)) |> Seq.mean
                m1-m2
            )

        let maxMeanAbundance = 
            normFrame
            |> Frame.mapRows (fun k s -> 
                let m1 = allColsOfIBait |> Seq.map (fun x -> s.GetAs<float>(x,nan)) |> Seq.mean
                let m2 = allColsOfIControl   |> Seq.map (fun x -> s.GetAs<float>(x,nan)) |> Seq.mean
                if m1 > m2 then m1 else m2
            )

        let meanAbundanceBait = 
            normFrame
            |> Frame.mapRows (fun k s -> 
                let m1 = allColsOfIBait |> Seq.map (fun x -> s.GetAs<float>(x,nan)) |> Seq.mean
                m1
            )

        let meanAbundanceControl = 
            normFrame
            |> Frame.mapRows (fun k s ->           
                let m2 = allColsOfIControl   |> Seq.map (fun x -> s.GetAs<float>(x,nan)) |> Seq.mean
                m2
            )    
        let stdev = 
            normFrame
            |> Frame.sliceCols allColsOfIBait 
            |> Frame.transpose
            |> Stats.stdDev
        let stdevControl = 
            normFrame
            |> Frame.sliceCols allColsOfIControl 
            |> Frame.transpose
            |> Stats.stdDev
        let qValue = 
            [|res.PosSigBioitem;res.NegSigBioitem;res.NonSigBioitem|]
            |> Array.concat
            |> Array.map (fun x -> x.ID,x.QValue)
            |> Series.ofObservations

        let IsCandidate = 
            [|res.PosSigBioitem;res.NegSigBioitem;res.NonSigBioitem|]
            |> Array.concat
            |> Array.map (fun x -> x.ID,x.QValue < fdr )
            |> Series.ofObservations
        
        let finPlot = 
            normFrame
            |> Frame.addCol "MeanR" fc
            |> Frame.addCol "MeanAbundance" maxMeanAbundance
            |> Frame.addCol "StDevR" stdev
            |> Frame.addCol "qValue" qValue
            |> Frame.addCol "missingValuesCondition" missingValuesCondition
            |> Frame.addCol "missingValuesControl" missingValuesControl
            |> Frame.addCol "IsCandidate" IsCandidate
            |> Frame.addCol "ProtName" 
                (allDataForAnalysis 
                |> Frame.mapRows (fun k s -> 
                    [|s.GetAs<string>("Synonym_Phytozome_61","");s.GetAs<string>("GeneName_Phytozome_61","");k|] 
                    |> String.concat ";"
                    |> fun x -> x.Replace('|',';').Split(';')
                    |> Array.distinct
                    |> Array.filter (fun x -> x <> "")
                    |> String.concat ";"
                    )   
                )//|> Array.distinct |> String.concat ";"))//(allDataForAnalysis.GetColumn<string>("Synonym"))   
        
        let fin = 
            normFrame
            |> Frame.addCol "log2FC_Bait_Minus_Control" fc
            |> Frame.addCol "MaxMeanLog2Abundance" maxMeanAbundance
            |> Frame.addCol "MeanLog2Abundance_Bait" meanAbundanceBait
            |> Frame.addCol "StDev_Bait" stdev
            |> Frame.addCol "MeanLog2Abundance_Control" meanAbundanceControl
            |> Frame.addCol "StDev_Control" stdevControl
            |> Frame.addCol "qValue" qValue
            |> Frame.addCol "MissingValues_Bait" missingValuesCondition
            |> Frame.addCol "MissingValues_Control" missingValuesControl
            |> Frame.addCol "IsSignificant" IsCandidate
            
        let toSave = Frame.join JoinKind.Outer fin (Frame.sliceCols ["Synonym"; "MapMan";"MapManDescription"; "GO"; "GODescription";"Localization";"Synonym_Phytozome_61";"GeneName_Phytozome_61"; "Description_Phytozome_61"] allDataForAnalysis)    
        
        /////////////////////////////////////////////////////
        // Save Results
        /////////////////////////////////////////////////////
        let outP = System.IO.Path.Combine([|runID;baitGroup|])
        System.IO.Directory.CreateDirectory outP
        |> ignore

        abundanceChart
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/1_Norm_PreNormGlobalAbundanceChart.html")
        corrFChart
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/2_Norm_CorrectionfactorChart.html")
        corrAbundanceChart
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/3_Norm_AfterNormGlobalAbundanceChart.html")
        preImpChart
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/4_Imputation_preImpChart.html")
        afterImpChart
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/5_Imputation_afterImpChart.html")
        preNormChart
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/6_QuantilNorm_PreNormChart.html")
        afterNormChart
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/7_QuantilNorm_AfterNormChart.html")
        createSAMChart res
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/8_Testing_SAM.html")
        plotHisto finPlot
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/9_Testing_DifferencesHistogram.html")
        plotMA [] finPlot // ["CDJ5";"LON";"PSBQLD";"PGRL1"]
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/10_Testing_MAPlot.html")
        plotVulcano [] finPlot // ["CDJ5";"LON";"PSBQLD";"PGRL1"]
        |> Chart.withTemplate ChartTemplates.lightMirrored
        |> Chart.saveHtml (outP + @"/11_Testing_VulcanoPlot.html")
        toSave.SaveCsv(outP + @"/Analysis_complete.tsv",includeRowKeys=true,keyNames=["Protein"],separator='\t')
        System.IO.File.WriteAllLines(outP + @"/params.txt",args)
        printfn "Finished Analysis! results can be found at:%s" outP 

    ////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////
    ////////////////////////////////////////
    // Execute Analysis
    ////////////////////////////////////////
    ///////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    let allDataForAnalysis :Frame<string,string> = 
        Frame.ReadCsv(path = outPathA, hasHeaders=true, separators="\t")
        |> Frame.indexRows columnHeader 

    let quantDataForAnalysis :Frame<string,string> = 
        let schema = 
            Frame.ReadCsv(path = outPathA, hasHeaders=true, separators="\t").ColumnKeys
            |> Seq.filter (fun x -> x.Contains "Intensity")
            |> Seq.map (fun x -> sprintf "%s=float" x)
            |> String.concat ","
        Frame.ReadCsv(path = outPathA, hasHeaders=true, separators="\t",schema= schema)
        |> Frame.indexRows columnHeader 
        |> Frame.mapValues (fun x -> if x > 0. then x else nan)

    let samplesGroupedByBioRepGroup = 
        allSamples
        |> Array.map (fun x -> 
            x, getBioRepGroup x)
        |> Array.groupBy snd
        |> Array.map (fun (x,y) -> 
            {|
            BaitGroup = x 
            QNodes = 
                y 
                |> Array.map fst 
            |}
            )
        
    let allBaitGroups = 
        samplesGroupedByBioRepGroup
        |> Array.filter (fun x -> x.BaitGroup <> "control")

    let allControlSamples = 
        samplesGroupedByBioRepGroup
        |> Array.filter (fun x ->  x.BaitGroup = "control")
        |> Array.map (fun x -> x.QNodes) 
        |> Array.concat

    allBaitGroups
    |> Array.iter (fun g -> doAnalysis allDataForAnalysis quantDataForAnalysis allControlSamples g.BaitGroup g.QNodes)    

    printfn "Done!" 




    0 // return an integer exit code